# =============================================================================
# @file    __version__.py
# @brief   CASICS Spiral version info
# @author  Michael Hucka <mhucka@caltech.edu>
# @license Please see the file named LICENSE in the project directory
# @website https://github.com/casics/spiral
# =============================================================================

__version__     = '1.1.0'
__title__       = 'Spiral'
__description__ = 'SPlitters for IdentifieRs: A Library'
__url__         = 'https://github.com/casics/spiral'
__author__      = 'Michael Hucka <mhucka@caltech.edu>'
__email__       = 'casics-team@googlegroups.com'
__license__     = 'GNU General Public License, version 3.0'
__copyright__   = 'Copyright (C) 2015-2017 by the California Institute of Technology'
__citation__    = 'Hucka, (2018). Spiral: splitters for identifiers in source code files. Journal of Open Source Software, 3(24), 653, https://doi.org/10.21105/joss.00653'
