#!/usr/bin/env python
# -*- encoding: utf-8 -*-

NOUN_STOPWORDS = {
    "Foo", "Bar", "foo", "bar", "instance", "object", "set", "map", "list", "type", "name", "value", "util", "utils"
}

VERB_STOPWORDS = {
    "get", "set", "new", "create",
}